﻿using System;
using System.Collections.Generic;
using Ozeki.VoIP;

namespace SIP_Message_Modification
{
    class Program
    {
        static ISoftPhone softphone;
        static IPhoneLine phoneLine;
        static SIP_CustomMessageManipulator sipMessageManipulator;  // creates a custom sip message manipulator object

        private static void Main(string[] args)
        {
            // Create a softphone object with RTP port range 5000-10000
            softphone = SoftPhoneFactory.CreateSoftPhone(5000, 10000);

            // SIP account registration data, (supplied by your VoIP service provider)
            var registrationRequired = true;
            var userName = "sipusername";
            var displayName = "sipdisplayname";
            var authenticationId = "authenticationid";
            var registerPassword = "Password";
            var domainHost = "pbxip.voipprovider.com";
            var domainPort = 5060;

            sipMessageManipulator = new SIP_CustomMessageManipulator(); // initializes the custom message manipulator

            var account = new SIPAccount(registrationRequired, displayName, userName, authenticationId, registerPassword, domainHost, domainPort);

            // Send SIP regitration request
            RegisterAccount(account);

            // Prevents the termination of the application
            Console.ReadLine();
        }

        static void RegisterAccount(SIPAccount account)
        {
            try
            {
                phoneLine = softphone.CreatePhoneLine(account);
                phoneLine.RegistrationStateChanged += sipAccount_RegStateChanged;
                softphone.RegisterPhoneLine(phoneLine);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error during SIP registration: " + ex);
            }
        }

        private static void sipAccount_RegStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            if (e.State == RegState.NotRegistered  || e.State == RegState.Error)
                Console.WriteLine("Registration failed!");

            if (e.State == RegState.RegistrationSucceeded)
                Console.WriteLine("Registration succeeded - Online!");
        }
    }

    class SIP_CustomMessageManipulator : ISIPMessageManipulator    // the class to implement a custom message manipulator needs to use the ISIPMessageManipulator interface
    {
        public string ModifyIncomingMessage(MessageModifierInfo incomingMessageInfo)  // modifies the incoming SIP message, if set
        {
            return incomingMessageInfo.Message; // gives back the message without any modification
        }

        public string ModifyOutgoingMessage(MessageModifierInfo outgoingMessageInfo)    // modifies the outgoing sip message, if set
        {
            return outgoingMessageInfo.Message.Replace("Ozeki VoIP SIP SDK", "Ozeki User-Agent");   // modifies the message's part to the set string
        }

        public IEnumerable<PreparedExtensionHeader> PrepareAdditionalHeaders(MessageModifierInfo outgoingMessageInfo)   // you can add additional headers to the SIP message
        {
            return null;    // no additional headers has been added to the sip message
        }
    }
}
